{% extends "admin/change_list.html" %}
{% load admin_list %}
{% load i18n %}
{% load treenode_admin %}

{% block object-tools-items %}
    {{ block.super }}
    {% if import_export_enabled %}
        <li><a href="import/" class="button">Import</a></li>
        <li><a href="export/" class="button">Export</a></li>
    {% endif %}
{% endblock %}

{% block result_list %}
  {% if action_form and actions_on_top and cl.show_admin_actions %}
    {% admin_actions %}
  {% endif %}

  {% tree_result_list cl %}

  {% if action_form and actions_on_bottom and cl.show_admin_actions %}
    {% admin_actions %}
  {% endif %}

{% endblock %}