from setuptools import setup, find_packages

setup(
    name='django-fast-treenode',
    version='3.0.3',
    description='Treenode Framework for supporting tree (hierarchical) data structure in Django projects',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='Timur Kady',
    author_email='timurkady@yandex.com',
    url='https://django-fast-treenode.readthedocs.io/',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    license='MIT',
    license_files=['LICENSE'],
    install_requires=[
        'Django>=4.0',
        'msgpack>=1.0.0',
        'openpyxl>=3.0.0',
        'pyyaml>=5.1',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
        'Framework :: Django',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
        'Framework :: Django :: 5.1',
        'Framework :: Django :: 5.2',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.9',
)

